#include <math.h>
#include "bin_deg.h"

// 定義すると、絶対値がこの値未満のsin値は0とする
// TODO:cosも同様の扱いをするため、sin=1, -1に近い値も同様の措置？
#define ABS_MIN_LIMIT   0.00001

double _bin_deg_sin_table[BIN_DEG_RESOLUTION];
//double _bin_deg_cos_table[BIN_DEG_RESOLUTION];
double _bin_deg_tan_table[BIN_DEG_RESOLUTION];
int _bin_deg_atan_table[BIN_DEG_ATAN_RESOLUTION+1];


int bin_deg_init(void){
    bin_deg_t d;
    int i;
    double r;
    for (d = 0 ; d < BIN_DEG_RESOLUTION ; d++){
        r = BinDeg2Rad(d);
        _bin_deg_sin_table[d] = sin(r);
#ifdef ABS_MIN_LIMIT        
        if (fabs(_bin_deg_sin_table[d]) < ABS_MIN_LIMIT){
            _bin_deg_sin_table[d] = 0;
        }
#endif        
        //        _bin_deg_cos_table[d] = cos(r);
        _bin_deg_tan_table[d] = tan(r);
    }
    for (i = 0 ; i <= BIN_DEG_ATAN_RESOLUTION ; i++){
        r = atan((double)i/BIN_DEG_ATAN_RESOLUTION);
        _bin_deg_atan_table[i] = Rad2BinDeg(r);
        //        printf("i=%d/%d r=%.5f d=%d\n", i, BIN_DEG_ATAN_RESOLUTION, r, _bin_deg_atan_table[i]);
    }
    //       exit(1);

    return 0;
}


//#define BinDegAtanLookup(y, x) ((int)(y/x*BIN_DEG_ATAN_RESOLUTION+0.5))
#define BinDegAtanLookup(y, x) ((int)(y/x*BIN_DEG_ATAN_RESOLUTION))

int bin_deg_atan2(double y, double x){
    bin_deg_t d;
    //    printf("%.2f, %.f2\n", x, y);
    if (y == 0){
        if (x < 0){
            return BIN_DEG_RANGLE*2;
        }
        // !!! x==0, y==0のときBIN_DEG_RANGLEを返す。これは仕様とする
        else {
            return 0;
        }
    }
    else if (x == 0){
        if (y < 0){
            return BIN_DEG_RANGLE*3;
        }
        // y==0の場合は上で処理済みなので、ここではy>0
        else {
            return BIN_DEG_RANGLE;
        }
    }
    // x!=0, y!=0
    else {
        if (x > 0){
            if (y > 0){
                if (y < x){
                    return _bin_deg_atan_table[BinDegAtanLookup(y, x)];
                }
                else {
                    d = _bin_deg_atan_table[BinDegAtanLookup(x, y)];
                    return BinDegComplementAngle(d);
                }
            }
            else { // y < 0
                if (-y < x){
                    d = _bin_deg_atan_table[BinDegAtanLookup(-y, x)];
                    return BinDegNegativeAngle(d);
                }
                else {
                    d = _bin_deg_atan_table[BinDegAtanLookup(x, -y)];
                    return BinDegNegativeAngle(BinDegComplementAngle(d));
                }
            }
        }
        else {
            if (y > 0){
                if (y < -x){
                    d = _bin_deg_atan_table[BinDegAtanLookup(y, -x)];
                    return BinDegSupplementAngle(d);
                }
                else {
                    d = _bin_deg_atan_table[BinDegAtanLookup(-x, y)];
                    return BinDegSupplementAngle(BinDegComplementAngle(d));
                }
            }
            else {
                if (-y < -x){
                    d = _bin_deg_atan_table[BinDegAtanLookup(-y, -x)];
                    return BinDegNegativeAngle(BinDegSupplementAngle(d));
                }
                else {
                    d = _bin_deg_atan_table[BinDegAtanLookup(-x, -y)];
                    return BinDegNegativeAngle(BinDegSupplementAngle(BinDegComplementAngle(d)));
                }
                //                return AddBinDeg(Rad2BinDeg(atan(y/x)), BIN_DEG_RESOLUTION>>1);
            }
        }
    }
}

