#include "config.h"
#include <stdio.h>
#include <string.h>
#include "param.h"
#include "layout.h"
#include "cell.h"

static cell_t cells[CELL_YNUM][CELL_XNUM];

void cell_init(void){
}

void setup_cell(void){
    memset(cells, 0, sizeof(cells));
}

void cell_iterator(cell_callback_t callback){
    int x, y;
    for (y = 0 ; y < CELL_YNUM ; y++){
        for (x = 0 ; x < CELL_XNUM ; x++){
            callback(x, y, &cells[y][x]);
        }
    }
}

void add_cell_unit_count(int cx, int cy, int val){
    //    printf("(%d, %d)\n", cx, cy);
    assert(cx>=0);
    assert(cy>=0);
    assert(cx<CELL_XNUM);
    assert(cy<CELL_YNUM);
    cells[cy][cx].snake_unit_count += val;
}

int get_cell_unit_count(int cx, int cy){
    assert(cx>=0);
    assert(cy>=0);
    assert(cx<CELL_XNUM);
    assert(cy<CELL_YNUM);
    return cells[cy][cx].snake_unit_count;
}

void add_neighbor_cell_unit_count(int cx, int cy, int val){
    int x, y;
    for (y = cy-1; y <= cy+1; y++){
        for (x = cx-1; x <= cx+1; x++){
            add_cell_unit_count(x, y, val);
        }
    }
}

int sum_neighbor_cell_unit_count(int cx, int cy){
    int x, y, sum=0;
    for (y = cy-1; y <= cy+1; y++){
        for (x = cx-1; x <= cx+1; x++){
            sum += get_cell_unit_count(x, y);
        }
    }
    return sum;
}





void set_cell_head(int x, int y){
    set_cell_flag(x, y, FLAG_HEAD);
}
void reset_cell_head(int x, int y){
    reset_cell_flag(x, y, FLAG_HEAD);
}
int get_cell_head(int x, int y){
    return is_cell_flag(x, y, FLAG_HEAD);
}
void set_cell_rat(int x, int y){
    set_cell_flag(x, y, FLAG_RAT);
}
void reset_cell_rat(int x, int y){
    reset_cell_flag(x, y, FLAG_RAT);
}
int get_cell_rat(int x, int y){
    return is_cell_flag(x, y, FLAG_RAT);
}

void set_cell_flag(int cx, int cy, int flag){
    assert(cx>=0);
    assert(cy>=0);
    assert(cx<CELL_XNUM);
    assert(cy<CELL_YNUM);
    cells[cy][cx].flags |= flag;
}
void reset_cell_flag(int cx, int cy, int flag){
    assert(cx>=0);
    assert(cy>=0);
    assert(cx<CELL_XNUM);
    assert(cy<CELL_YNUM);
    cells[cy][cx].flags &= ~flag;
}
int is_cell_flag(int cx, int cy, int flag){
    assert(cx>=0);
    assert(cy>=0);
    assert(cx<CELL_XNUM);
    assert(cy<CELL_YNUM);
    if (cells[cy][cx].flags & flag) return 1;
    return 0;
}

void set_neighbor_cell_flag(int cx, int cy, int flag){
    int x, y;
    for (y = cy-1; y <= cy+1; y++){
        for (x = cx-1; x <= cx+1; x++){
            set_cell_flag(x, y, flag);
        }
    }
}
void reset_neighbor_cell_flag(int cx, int cy, int flag){
    int x, y;
    for (y = cy-1; y <= cy+1; y++){
        for (x = cx-1; x <= cx+1; x++){
            reset_cell_flag(x, y, flag);
        }
    }
}


int count_neighbor_cell_flag(int cx, int cy, int flag){
    int x, y, sum=0;
    for (y = cy-1; y <= cy+1; y++){
        for (x = cx-1; x <= cx+1; x++){
            sum += is_cell_flag(x, y, flag);
        }
    }
    return sum;
}

