#ifndef CELL_H
#define CELL_H
#include "config.h"

typedef struct {
    int snake_unit_count;
    unsigned int flags;
} cell_t;

#define FLAG_HEAD 1
#define FLAG_RAT (1 << 1)


typedef void (*cell_callback_t)(int, int, cell_t*);

void cell_init(void);
void setup_cell(void);
void cell_iterator(cell_callback_t callback);
void add_cell_unit_count(int x, int y, int val);
int get_cell_unit_count(int x, int y);
void add_neighbor_cell_unit_count(int cx, int cy, int count);
int sum_neighbor_cell_unit_count(int cx, int cy);
void set_cell_flag(int cx, int cy, int flag);
void reset_cell_flag(int cx, int cy, int flag);
int is_cell_flag(int cx, int cy, int flag);
void set_neighbor_cell_flag(int cx, int cy, int flag);
void reset_neighbor_cell_flag(int cx, int cy, int flag);
int count_neighbor_cell_flag(int cx, int cy, int flag);

#define X2Cx(x) ((int)(x)/ CELL_SIZE)
#define Y2Cy(y) (X2Cx(y))

#define AddCellUnitCount(x, y, val) \
    (add_neighbor_cell_unit_count(X2Cx(x), Y2Cy(y), val))
#define GetCellUnitCount(x, y) \
    (sum_neighbor_cell_unit_count(X2Cx(x), Y2Cy(y)))

#define SetCellHeadFlag(x, y) \
    (set_neighbor_cell_flag(X2Cx(x), Y2Cy(y), FLAG_HEAD))
#define ResetCellHeadFlag(x, y) \
    (reset_neighbor_cell_flag(X2Cx(x), Y2Cy(y), FLAG_HEAD))
#define CountCellHeadFlag(x, y) \
    (count_neighbor_cell_flag(X2Cx(x), Y2Cy(y), FLAG_HEAD))

#define SetCellRatFlag(x, y)                               \
    (set_neighbor_cell_flag(X2Cx(x), Y2Cy(y), FLAG_RAT))
#define ResetCellRatFlag(x, y) \
    (reset_neighbor_cell_flag(X2Cx(x), Y2Cy(y), FLAG_RAT))
#define CountCellRatFlag(x, y) \
    (count_neighbor_cell_flag(X2Cx(x), Y2Cy(y), FLAG_RAT))
    
















/*

  #define AddCellUnitCount(cx,   cy,   val)     \
    add_cell_unit_count(cx,   cy,   val), \
    add_cell_unit_count(cx-1, cy,   val), \
    add_cell_unit_count(cx,   cy-1, val), \
    add_cell_unit_count(cx-1, cy-1, val) 

#define GetCellUnitCount(cx, cy) \
    (get_cell_unit_count(cx,   cy  ) + \
     get_cell_unit_count(cx-1, cy  ) + \
     get_cell_unit_count(cx,   cy-1) + \
     get_cell_unit_count(cx-1, cy-1))

#define SetCellHead(cx,   cy)   \
    set_cell_head(cx,   cy),    \
    set_cell_head(cx-1, cy),    \
    set_cell_head(cx,   cy-1),  \
    set_cell_head(cx-1, cy-1)
#define ResetCellHead(cx,   cy)   \
    reset_cell_head(cx,   cy),    \
    reset_cell_head(cx-1, cy),    \
    reset_cell_head(cx,   cy-1),  \
    reset_cell_head(cx-1, cy-1)
#define GetCellHeadCount(cx, cy) \
    (get_cell_head(cx,   cy  ) + \
     get_cell_head(cx-1, cy  ) + \
     get_cell_head(cx,   cy-1) + \
     get_cell_head(cx-1, cy-1))

#define SetCellRat(cx,   cy)   \
    set_cell_rat(cx,   cy),    \
    set_cell_rat(cx-1, cy),    \
    set_cell_rat(cx,   cy-1),  \
    set_cell_rat(cx-1, cy-1)
#define ResetCellRat(cx,   cy)   \
    reset_cell_rat(cx,   cy),    \
    reset_cell_rat(cx-1, cy),    \
    reset_cell_rat(cx,   cy-1),  \
    reset_cell_rat(cx-1, cy-1)
#define GetCellRatCount(cx, cy) \
    (get_cell_rat(cx,   cy  ) + \
     get_cell_rat(cx-1, cy  ) + \
     get_cell_rat(cx,   cy-1) + \
     get_cell_rat(cx-1, cy-1))
    
    


void set_cell_head(int x, int y);
void reset_cell_head(int x, int y);
int get_cell_head(int x, int y);
void set_cell_rat(int x, int y);
void reset_cell_rat(int x, int y);
int get_cell_rat(int x, int y);
*/

#endif
