#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <EzGraph.h>
#include "draw.h"
#include "game.h"
#include "snake.h"
#include "cell.h"
#include "obj.h"
#include "font.h"


static char *gameover_str = "げーむおーばー";
static font_id_t *gameover_fontseq;
static int draw_cell_enabled = 0;

static font_id_t score_fontseq[SCORE_LEN+1];
static font_id_t hiscore_fontseq[SCORE_LEN+1];

static char *score_caption_str = "すこあ";
static font_id_t *score_caption_fontseq;
static char *hiscore_caption_str = "はいすこあ";
static font_id_t *hiscore_caption_fontseq;


void draw_cells(void);
void draw_objs(void);
void draw_status(void);
void draw_snake(void);


void draw_init(void){
    score_caption_fontseq = alloc_fontseq(score_caption_str);
    hiscore_caption_fontseq = alloc_fontseq(hiscore_caption_str);
    gameover_fontseq = alloc_fontseq(gameover_str);
    disable_draw_cell();

}


void setup_draw_screen(void){
    update_draw_score();
}

void draw_screen(void){
    // セル表示モードのときは背景塗りつぶさない
    if (!draw_cell_enabled){
        EzSetColorByRGB(120, 40, 0);    // フィールド背景色
        EzFillBoxB(FIELD_LX, FIELD_TY, FIELD_RX, FIELD_BY);
    }
    EzSetColorByRGB(180, 180, 180); 
    EzDrawBoxB(FIELD_LX, FIELD_TY, FIELD_RX, FIELD_BY);
    if (draw_cell_enabled){
        draw_cells();
    }
    //    puts("draw screen");
    draw_objs();
    draw_snake();
    if (get_game_state() == gs_dying){
        draw_fontseq((FIELD_WIDTH-16*7)/2, 228, gameover_fontseq);
    }
    draw_status();
    EzShowBuffer();
}


void draw_cell_callback(int cx, int cy, cell_t *cell){
    int x = cx*CELL_SIZE;
    int y = cy*CELL_SIZE;
    int c;

    EzSetColorByRGB(255,255,255);
    if (cell->snake_unit_count > 0){
        c = cell->snake_unit_count*32+128;
        c = (c > 255) ? 255 : c;
        EzSetColorByRGB(c, c, c);
        EzFillBoxB(FIELD_LX+x,
                   FIELD_TY+y,
                   FIELD_LX+x+CELL_SIZE-2,
                   FIELD_TY+y+CELL_SIZE-2);
    }
    if (is_cell_flag(cx, cy, FLAG_HEAD)){
        EzSetColorByRGB(255, 0, 0);
        EzFillBoxB(FIELD_LX+x,
                   FIELD_TY+y,
                   FIELD_LX+x+CELL_SIZE-2,
                   FIELD_TY+y+CELL_SIZE-2);
    }
    if (is_cell_flag(cx, cy, FLAG_RAT)){
        EzSetColorByRGB(255, 0, 255);
        EzFillBoxB(FIELD_LX+x,
                   FIELD_TY+y,
                   FIELD_LX+x+CELL_SIZE-2,
                   FIELD_TY+y+CELL_SIZE-2);
    }
    EzSetColorByRGB(255,255,255);
    EzDrawLineB(FIELD_LX+x,
                FIELD_TY+y+CELL_SIZE-1,
                FIELD_LX+x+CELL_SIZE-1,
                FIELD_TY+y+CELL_SIZE-1);

    EzDrawLineB(FIELD_LX+x,
                FIELD_TY+y+CELL_SIZE-1,
                FIELD_LX+x+CELL_SIZE-1,
                FIELD_TY+y+CELL_SIZE-1);
    EzDrawLineB(FIELD_LX+x+CELL_SIZE-1,
                FIELD_TY+y,
                FIELD_LX+x+CELL_SIZE-1,
                FIELD_TY+y+CELL_SIZE-1);
}

void draw_cells(void){
    cell_iterator(draw_cell_callback);
}

void draw_snake(void){
    snake_iterator(draw_snake_callback, draw_snake_head_callback);
}


void draw_snake_callback(snake_unit_t *unit){
    //    printf("callback unit(%d, %d), %d)\n", unit->x, unit->y, unit->deg);
    EzSetColorByRGB(160, 160, 0);
    EzFillCircleB((int)(unit->x)+FIELD_LX, (int)(unit->y)+FIELD_TY,
                  SNAKE_UNIT_R-2);
}

void draw_snake_head_callback(snake_unit_t *unit){
    int x, y;
    bin_deg_t deg;
    double ax, ay;
    int ac, ar;

    //    EzSetColorByRGB(255, 0, 0);
    //    EzDrawCircleB((int)(unit->x)+FIELD_LX, (int)(unit->y)+FIELD_TY,
    //                  SNAKE_UNIT_R);

    EzSetColorByRGB(160, 160, 0);
    deg = BinDegAdd(unit->deg, BIN_DEG_SANGLE);
    x = BinDegCos(deg) * 8;
    y = BinDegSin(deg) * 8;
    EzFillCircleB(x+(int)(unit->x)+FIELD_LX, y+(int)(unit->y)+FIELD_TY,
                  SNAKE_UNIT_R);
    deg = unit->deg;
    x = BinDegCos(deg) * 4;
    y = BinDegSin(deg) * 4;
    EzFillCircleB(x+(int)(unit->x)+FIELD_LX, y+(int)(unit->y)+FIELD_TY,
                  SNAKE_UNIT_R-4);
    
    EzSetColorByRGB(0, 0, 0);
    deg = BinDegSub(unit->deg, SNAKE_EYE_DEG);
    x = BinDegCosN(deg) * (SNAKE_SPEED-4);
    y = BinDegSinN(deg) * (SNAKE_SPEED-4);
    EzFillCircleB(x+(int)(unit->x)+FIELD_LX, y+(int)(unit->y)+FIELD_TY,
                  4);
    deg = BinDegAdd(unit->deg, SNAKE_EYE_DEG);
    x = BinDegCosN(deg) * (SNAKE_SPEED-4);
    y = BinDegSinN(deg) * (SNAKE_SPEED-4);
    EzFillCircleB(x+(int)(unit->x)+FIELD_LX, y+(int)(unit->y)+FIELD_TY,
                  4);

    if ((ac = get_snake_attack_count()) > 0){
        get_snake_attack_pos(&ax, &ay, &ar);
        EzSetColorByRGB(255, 0, 0);
        EzDrawCircleB(ax+FIELD_LX, ay+FIELD_TY, ar);
    }
    EzSetColorByRGB(0, 0, 0);
}

void draw_objs(void){
    obj_iterator_draw();
}

void enable_draw_cell(void){
    puts("enable draw cell!!!!!");
    draw_cell_enabled = 1;
}
void disable_draw_cell(void){
    puts("disable draw cell!!!!!");
    draw_cell_enabled = 0;
}
int is_draw_cell_enabled(void){
    return draw_cell_enabled;
}

void draw_status(void){
    draw_fontseq(STATUS_LX,  STATUS_TY,             hiscore_caption_fontseq);
    draw_fontseq(STATUS_LX,  STATUS_TY+LINE_SKIP,   hiscore_fontseq);
    draw_fontseq(STATUS_LX,  STATUS_TY+LINE_SKIP*2, score_caption_fontseq);
    draw_fontseq(STATUS_LX,  STATUS_TY+LINE_SKIP*3, score_fontseq);
}
    

void update_draw_score(void){
    uint2fontseq(get_score(), SCORE_LEN, score_fontseq);
    uint2fontseq(get_hiscore(), SCORE_LEN, hiscore_fontseq);
    printf("sc:%d hsc:%d\n", get_score(), get_hiscore());
}

void draw_marker_circle(double x, double y, int r){
    EzSetColorByRGB(255, 0, 0);
    EzDrawCircleB(x+FIELD_LX, y+FIELD_TY, r);
}
