#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <EzGraph.h>
#include "param.h"
#include "layout.h"
#include "game.h"
#include "snake.h"
#include "cell.h"
#include "draw.h"
#include "obj.h"
#include "bin_deg.h"


static snake_unit_t units[SNAKE_MAX_LEN];
static int head_ix, tail_ix;
static snake_unit_t *head, *new_head, *tail;
static int snake_len;
static int grow_count;
// 領域外に出そうなときに自動旋回を試みる際の単位角度
bin_deg_t border_turn_deg;

// 閉空間を形成(i.e. 頭が体に接した)
static int is_closed;
static int attack_count;
static double attack_x, attack_y;

#define NextIx(ix) (((ix)+1) % SNAKE_MAX_LEN)
#define PrevIx(ix) (((ix)+SNAKE_MAX_LEN-1) % SNAKE_MAX_LEN)
#define AddIx(ix, val) (((ix)+val) % SNAKE_MAX_LEN)
#define SubIx(ix, val) (((ix)+SNAKE_MAX_LEN-val) % SNAKE_MAX_LEN)
#define NextHeadIx NextIx(head_ix)
#define NextHead units[NextIx];

#define AddCount(unit) \
    (AddCellUnitCount((unit)->x, (unit)->y, 1))
#define SubCount(unit) \
    (AddCellUnitCount((unit)->x, (unit)->y, -1))
#define GetCount(unit) \
    (GetCellUnitCount((unit)->x, (unit)->y))

static void setup_snake(int x, int y, bin_deg_t deg);
static void proceed_head(bin_deg_t turn_deg);
static void proceed_tail(void);
static void setup_tail(void);
void snake_attack_callback(obj_t *obj);

void snake_init(void){
}

void setup_snake_classic(int x, int y, bin_deg_t deg){
    snake_len = SNAKE_LEN_CLASSIC_DEFAULT;
    border_turn_deg = BIN_DEG_RANGLE;
    setup_snake(x, y, deg);
}
void setup_snake_uroboros(int x, int y, bin_deg_t deg){
    snake_len = SNAKE_LEN_UROBOROS_DEFAULT;
    border_turn_deg = 1;
    setup_snake(x, y, deg);
    printf("setup snake %d\n", border_turn_deg);
    is_closed = 0;
    attack_count = 0;
}


static void setup_snake(int x, int y, bin_deg_t deg){
    int i;
    grow_count = 0;

    head_ix = tail_ix = 0;

    head = &units[head_ix];
    head->deg = deg;
    head->x = x;    
    head->y = y;    
    AddCount(head);
    printf("setup (%f, %f) %d\n", head->x, head->y, head->deg);
    //    head_ix = NextHeadIx;
    for (i = 0; i < snake_len-1; i++){
        proceed_head(0);
    }
    setup_tail();
}

void snake_iterator(snake_callback_t callback,
                    snake_callback_t head_callback)
{
    int ix;
    assert(callback != NULL);
    //    printf("snake iterator(%d-%d)\n", head_ix, tail_ix);
    // tailからheadの一つ前までコールバック
    for (ix = tail_ix; ix != head_ix; ix = NextIx(ix)){
        //        printf("call %d(%d-%d)\n", ix, head_ix, tail_ix);
        callback(&units[ix]);
    }
    // head専用コールバックが指定されていればそちらを呼ぶ
    if (head_callback != NULL){
        head_callback(head);
    }
    else {
        callback(head);
    }
}

void proceed_snake(bin_deg_t turn_deg){
    // 伸ばした頭の衝突判定時に前ターンでのしっぽが引っかからないように
    // 先にしっぽを縮めておく
    assert(snake_len > 1);// へびの長さは常に2以上はあると想定
    if (grow_count > 0){
        printf("grow snake %d->%d\n", snake_len, snake_len+1);
        snake_len++;
        grow_count--;
    }
    else {
        proceed_tail();
    }
    proceed_head(turn_deg);
}

static void proceed_head(bin_deg_t turn_deg){
    bin_deg_t deg;

    if (attack_count>0) attack_count--;
    
    head_ix = NextIx(head_ix);
    new_head = &units[head_ix];
    deg = BinDegAdd(head->deg, turn_deg);
    new_head->x = head->x + BinDegCosN(deg) * SNAKE_SPEED;
    new_head->y = head->y + BinDegSinN(deg) * SNAKE_SPEED;
    while (new_head->x <= FIELD_BORDER_MARGIN ||
           new_head->x >= FIELD_WIDTH-FIELD_BORDER_MARGIN-1 ||
           new_head->y <= FIELD_BORDER_MARGIN ||
           new_head->y >= FIELD_HEIGHT-FIELD_BORDER_MARGIN-1)
    {
        printf("(%.0f,%.0f) %d\n",new_head->x, new_head->y, FIELD_BORDER_MARGIN); 
        //        printf("turn deg:%d %d\n", deg, turn_deg);
        deg = BinDegAdd(deg, border_turn_deg);
        //        printf("turn deg2:%d\n", deg);
        //        printf("turn deg:%d\n", deg);
        new_head->x = head->x + BinDegCosN(deg) * SNAKE_SPEED;
        new_head->y = head->y + BinDegSinN(deg) * SNAKE_SPEED;
    }
    AddCount(new_head);
    
    new_head->deg = deg;
    printf("(%.0f, %.0f), deg:%d turn:%d cos/sin(%f %f) (%f %f)\n", new_head->x, new_head->y, deg, turn_deg, BinDegCosN(deg), BinDegSinN(deg), new_head->x, new_head->y);

    head = new_head;

    obj_iterator(snake_attack_callback);
}

void snake_attack_callback(obj_t *obj){
    if (obj->type == ot_rat &&
        is_overlapped(obj->x, obj->y, attack_x, attack_y, SQ(SNAKE_ATTACK_R)))
    {
        obj->ctr2 = RAT_RESPPAWN_COUNT;
        obj->x = -100; // 死んでいる間はフィールド外においておく
        add_score(RAT_SCORE);
    }
}

static void proceed_tail(void){
    SubCount(tail);
    tail_ix = NextIx(tail_ix);
    setup_tail();
}

static void setup_tail(void){
    tail = &units[tail_ix];
}


int judge_snake_bump(void){
    int c = GetCount(head);
    printf("head count:%d\n", c);
    if (c > 9+6+3+12){
        is_closed = 1;
        return 1;
    }
    else {
        is_closed = 0;
        return 0;
    }
}

void set_snake_bite(void){
    SetCellHeadFlag(head->x, head->y);
}
void reset_snake_bite(void){
    ResetCellHeadFlag(head->x, head->y);
}
    
                     
void grow_snake(int count){
    grow_count += count;
}

int is_head_overlapped(double x, double y, int dist2){
    return is_overlapped(head->x, head->y, x, y, dist2);
}

void snake_attack(void){
    int ix, i;
    double gx, gy;
    int gnum;
    // とりあえず、攻撃中は再攻撃しない
    if (attack_count > 0) return;

    ix = head_ix;
    gx = head->x;
    gy = head->y;
    gnum = 1;
    // 頭直後のユニットは頭と重なっているので強制的にいくつか戻る
    for (i = 0 ; i < 4 ; i++){
        ix = PrevIx(ix);
        gx += units[ix].x;
        gy += units[ix].y;
        gnum++;
    }
    while (!is_overlapped(head->x, head->y, units[ix].x, units[ix].y,
                          (SNAKE_SPEED*2)*(SNAKE_SPEED*2)))
    {
        printf("prev ix:%d\n", ix);
        draw_marker_circle(units[ix].x, units[ix].y, 50);
        ix = PrevIx(ix);
        gx += units[ix].x;
        gy += units[ix].y;
        gnum++;
    }
    printf("attack_pos(%0.f, %0.f) (%0.f %0.f) gnum:%d\n", attack_x, attack_y, gx, gy, gnum);
    attack_x = gx / gnum;
    attack_y = gy / gnum;
    
    printf("snake attack!!\n");
    attack_count = 5;
}

int get_snake_attack_count(void){
    return attack_count;
}

void get_snake_attack_pos(double *x, double *y, int *r){
    *x = attack_x;
    *y = attack_y;
    *r = attack_count*20;
}
